/*
  Master Sation software ver 1.0 

  This sketch reads the data registered by the multi-modules inclinometer
  The software is coded and compliled with Arduino IDE https://www.arduino.cc/en/Main/Software
  
  Used electronic devices:
 
 - Arduino MEGA  https://store.arduino.cc/arduino-mega-2560-rev3
 - RS 485 board 
 - I2C RTC clock DS1307
 - I2C 0.96 in OLED display
 - SPI Micro SD card adapter 
 - Self-made interface board  
 
  created 10 May 2020 
  by Giuseppe Ruzza
  
  Department of Science and Technology 
  University of Sannio, Benevento, Italy
  */
                                                                                                                                                          

#include "RS485_protocol.h" // RS 485 communication protocol library http://www.gammon.com.au/forum/?id=11428
                                                                                                                                                                                                                                                                                                                          
////////////////////////////////////////// Oled functions

#include <Adafruit_GFX.h>      //OLED library https://github.com/adafruit/Adafruit-GFX-Library
#include <Adafruit_SSD1306.h>  //OLED library https://github.com/adafruit/Adafruit_SSD1306

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 oled(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

////////////////////////////////////////// SD card adapter functions

#include <SPI.h>  // SPI library
#include <SD.h>   // SD  library
const int chipSelect = 53; //Arduino Mega pin for SD card selection functions

////////////////////////////////////////// RTC clock functions

#include <Wire.h>     // I2C library
#include "RTClib.h"   // RTC library  https://github.com/adafruit/RTClib/blob/master/RTClib.h
RTC_DS1307 rtc;

///////////////////////////////////////////Comunication loop functions

void fWrite (const byte what)
  {
  Serial3.write(what);
  Serial3.flush();   
  }
  
int fAvailable ()
  {
  return Serial3.available ()> 0;  
  }

int fRead ()
  {
  return Serial3.read ();  
  }

//////////////////////////////////////variables

const byte ENABLE_PIN = 9; //RS485 board pin functions
//led ceck pin
int ledmodul1 = 31;
int ledmodul2 = 33;
int ledmodul3 = 35;
int ledmodul4 = 37;
int ledmodul5 = 39;
int read = 47;
int zerorun = 43;
int comando;
int a = 0;
int valoreint;
int prima =0;
int sdceck;
int Index1;
int Index2;
int Index3;
int Index4;
int Index5;
int Index6;
int ceck = 1;
int down1 = 0;
int down2 = 0;
int down3 = 0;
int down4 = 0;
int down5 = 0;
int delaybetween = 2000;
int LED = 45;          
int brightness = 0;   
int fadeAmount = 5;    
int misurazero = 0;    
int x = 1;
float meantemp;
float percentage;
float conversionFactor = 0.00006103515; //Conversion factor from accelerometric raw data to gravity data
float bx,by; // oled display variables

//Variables for zero position setting
float tiltzero1; 
float tiltzero2; 
float tiltzero3; 
float tiltzero4; 
float tiltzero5; 
float pitchzero1; 
float pitchzero2; 
float pitchzero3;
float pitchzero4; 
float pitchzero5; 
float rollzero1; 
float rollzero2; 
float rollzero3; 
float rollzero4; 
float rollzero5;

float cicleread = 20; //Number of  data cycle visualizzation
//data received variables
float Gravityx1, Gravityx2, Gravityx3, Gravityx4, Gravityx5;
float Gravityy1, Gravityy2, Gravityy3, Gravityy4, Gravityy5;
float Gravityz1, Gravityz2, Gravityz3, Gravityz4, Gravityz5;
float GravityxCal1, GravityxCal2, GravityxCal3, GravityxCal4, GravityxCal5;
float GravityyCal1, GravityyCal2, GravityyCal3, GravityyCal4, GravityyCal5;
float GravityzCal1, GravityzCal2, GravityzCal3, GravityzCal4, GravityzCal5;
float GravityxOff1, GravityxOff2, GravityxOff3, GravityxOff4, GravityxOff5;
float GravityyOff1, GravityyOff2, GravityyOff3, GravityyOff4, GravityyOff5;
float GravityzOff1, GravityzOff2, GravityzOff3, GravityzOff4, GravityzOff5;
//temperature value of each sensors
float temperature1, temperature2, temperature3, temperature4, temperature5;

//Lenght of tilt module for derive the displacement values
float lenghtmodule =  40.0; 

//Variables to check the total gravity of each sensor 
float TotalG1;
float TotalG2;
float TotalG3;
float TotalG4;
float TotalG5;

//Angular values and directions
float roll1;
float roll2;
float roll3;
float roll4;
float roll5;
float pitch1;
float pitch2;
float pitch3;
float pitch4;
float pitch5;
float tilt1;
float tilt2;
float tilt3;
float tilt4;
float tilt5;
float Direction1;
float Direction2;
float Direction3;
float Direction4;
float Direction5;
//Displacement of each module
float displacement1, displacement2, displacement3, displacement4, displacement5, totdisplacementx;
//Resultant of the displacement of the X axis 
float resultantx1, resultantx2, resultantx3, resultantx4, resultantx5;

void setup() {
  
  if(!oled.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3D for 128x64 OLED
    oled.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }

  rtc.begin(); //RTC clock begin 
  Serial.begin(9600); //Communication begin
  Serial3.begin(9600); // Communication begin
  //Pin setting
  pinMode(ledmodul1, OUTPUT);
  pinMode(LED, OUTPUT);
  pinMode(ledmodul2, OUTPUT);
  pinMode(ledmodul3, OUTPUT);
  pinMode(ledmodul4, OUTPUT);
  pinMode(ledmodul5, OUTPUT);
  pinMode(read, OUTPUT);
  pinMode(zerorun, OUTPUT);
  pinMode (ENABLE_PIN, OUTPUT);  
  digitalWrite (ENABLE_PIN, LOW); 
  digitalWrite(ledmodul1, LOW);
  digitalWrite(ledmodul2, LOW);
  digitalWrite(ledmodul3, LOW);
  digitalWrite(ledmodul4, LOW);
  digitalWrite(ledmodul5, LOW);
  digitalWrite(read, LOW);
  digitalWrite(zerorun, LOW);

 
}

void loop() {
 // Displaying time for 20 seconds
 for (int i = 0; i <= 20; i++) {
    oled.setTextSize(2);
    oled.setTextColor(SSD1306_WHITE);
    DateTime now = rtc.now();                                                                
    oled.clearDisplay();
    oled.setCursor(0,0);
    oled.print(now.hour(), DEC); 
    oled.print(":");
    oled.print(now.minute(), DEC);
    oled.print(":");
    oled.println(now.second(), DEC);
    oled.print(now.day(), DEC);
    oled.print("/");
    oled.print(now.month(), DEC);
    oled.print("/");
    oled.print(now.year(), DEC);   
    oled.display();
    delay(1000);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Send request and read data from the module 1

byte msg1 [] = { // Device 1
                 1    
               };

digitalWrite (ENABLE_PIN, HIGH); 
delay(1);
sendMsg (fWrite, msg1, sizeof msg1); //Send request to address module
digitalWrite (ENABLE_PIN, LOW);  // Disable sending

 for (int i=0; i<14; i++) {  //Receiving requested data

                      byte buf [9];
                      byte received = recvMsg (fAvailable, fRead, buf, sizeof buf);

                      if (received)
                      {
                        if (buf [0] != 1)
                          return; 
                        if(buf [0] == 1)
                        {
                          
                                temperature1    = (int)(((unsigned)buf [2] << 8) | buf [1]);
                                Gravityx1       = (int)(((unsigned)buf [4] << 8) | buf [3]);
                                Gravityy1       = (int)(((unsigned)buf [6] << 8) | buf [5]);
                                Gravityz1       = (int)(((unsigned)buf [8] << 8) | buf [7]);

                                //Derive temperature of module
                                temperature1    = (21.00 +(((float)temperature1)/8));
                                //Derive gravity value from the raw data
                                Gravityx1      *= conversionFactor;
                                Gravityy1      *= conversionFactor;
                                Gravityz1      *= conversionFactor;
                                //Apply the offset calibration
                                GravityxOff1 = Gravityx1 - (-0.065329);
                                GravityyOff1 = Gravityy1 - (-0.010816);
                                GravityzOff1 = Gravityz1 - (-0.066808);
                                GravityxCal1 =  1.000506 * GravityxOff1 + 0.006846 * GravityyOff1 - 0.008642 * GravityzOff1;
                                GravityyCal1 =  0.006846 * GravityxOff1 + 0.983629 * GravityyOff1 + 0.058866 * GravityzOff1;
                                GravityzCal1 = -0.008642 * GravityxOff1 + 0.058866 * GravityyOff1 + 0.936130 * GravityzOff1;
                                //Derive the total gravity vector
                                TotalG1 = sqrt ((GravityxCal1*GravityxCal1)+(GravityyCal1*GravityyCal1)+(GravityzCal1*GravityzCal1));
                     
                       ceck = 0;
                    
                      }
                      }
                   }
                   
 if (ceck == 0) //Displaing the preliminar value and the validity of the module sensor
 {
    down1 = 0;
    digitalWrite(ledmodul1, HIGH);
    oled.clearDisplay();
    oled.setCursor(0,0);
    oled.println("Module 1: OK");
    oled.setTextSize(1);
    oled.setTextColor(SSD1306_WHITE);
    oled.print ("Gx: ");
    oled.println(Gravityx1);
    oled.print ("Gy: ");
    oled.print(Gravityy1);
    oled.print("  ");
    oled.print("TG: ");
    oled.println(TotalG1);
    oled.print ("Gz: ");
    oled.println(Gravityz1);
    oled.display();
 } 
 
 oled.setTextSize(2);
 oled.setTextColor(SSD1306_WHITE);

 if (ceck == 1 || TotalG1 < 0.1 && TotalG1 > 1.1) 
 {
     down1 = 1;
     digitalWrite(ledmodul1, LOW);
     oled.clearDisplay();
     oled.setCursor(0,0);
     oled.print("Module 1: KO");
     oled.setTextSize(1);
     oled.setTextColor(SSD1306_WHITE);
     oled.print ("Gx: ");
     oled.println(Gravityx1);
     oled.print ("Gy: ");
     oled.print(Gravityy1);
     oled.print("  ");
     oled.print("TG: ");
     oled.println (TotalG1);
     oled.print ("Gz: ");
     oled.println(Gravityz1);
     oled.display();   
 } 

 ceck = 1;   
 delay(delaybetween); 

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Send request and read data from the module 2

byte msg2 [] = { 
                 2    
               };

digitalWrite (ENABLE_PIN, HIGH); 
delay(1);
sendMsg (fWrite, msg2, sizeof msg2);

digitalWrite (ENABLE_PIN, LOW);  

 for (int i=0; i<14; i++) {

                      byte buf [9];
                      byte received = recvMsg (fAvailable, fRead, buf, sizeof buf);

                      if (received)
                      {
                        if (buf [0] != 2)
                          return; 
                        if(buf [0] == 2)
                        {
                          
                                temperature2    = (int)(((unsigned)buf [2] << 8) | buf [1]);
                                Gravityx2       = (int)(((unsigned)buf [4] << 8) | buf [3]);
                                Gravityy2       = (int)(((unsigned)buf [6] << 8) | buf [5]);
                                Gravityz2       = (int)(((unsigned)buf [8] << 8) | buf [7]);

                                temperature2    = (21.00 +(((float)temperature2)/8));
                                
                                Gravityx2      *= conversionFactor;
                                Gravityy2      *= conversionFactor;
                                Gravityz2      *= conversionFactor;
                                
                                GravityxOff2 = Gravityx2 - (-0.083679);
                                GravityyOff2 = Gravityy2 - (-0.007864);
                                GravityzOff2 = Gravityz2 - (0.030250);

                                GravityxCal2 =  1.001324 * GravityxOff2 + 0.007998 * GravityyOff2 + 0.007558 * GravityzOff2;
                                GravityyCal2 =  0.007998 * GravityxOff2 + 1.003540 * GravityyOff2 - 0.008576 * GravityzOff2;
                                GravityzCal2 =  0.007558 * GravityxOff2 - 0.008576 * GravityyOff2 + 1.098995 * GravityzOff2;
                                
                                TotalG2 = sqrt ((GravityxCal2*GravityxCal2)+(GravityyCal2*GravityyCal2)+(GravityzCal2*GravityzCal2));
                                              
                                
                     
                       ceck = 0;
                    
                      }
                      }
                   }
 if (ceck == 0)
 {
    down1 = 0;
    digitalWrite(ledmodul2, HIGH);
    oled.clearDisplay();
    oled.setCursor(0,0);
    oled.println("Module 2: OK");
    oled.setTextSize(1);
    oled.setTextColor(SSD1306_WHITE);
    oled.print ("Gx: ");
    oled.println(Gravityx2);
    oled.print ("Gy: ");
    oled.print(Gravityy2);
    oled.print("  ");
    oled.print("TG: ");
    oled.println (TotalG2);
    oled.print ("Gz: ");
    oled.println(Gravityz2);
    oled.display();
 } 
 
 oled.setTextSize(2);
 oled.setTextColor(SSD1306_WHITE);

 if (ceck == 1 || TotalG2 < 0.5 && TotalG2 > 1.5) 
 {
     down1 = 1;
     digitalWrite(ledmodul2, LOW);
     oled.clearDisplay();
     oled.setCursor(0,0);
     oled.print("Module 2: KO");
     oled.setTextSize(1);
     oled.setTextColor(SSD1306_WHITE);
     oled.print ("Gx: ");
     oled.println(Gravityx2);
     oled.print ("Gy: ");
     oled.print(Gravityy2);
     oled.print("  ");
     oled.print("TG: ");
     oled.println (TotalG2);
     oled.print ("Gz: ");
     oled.println(Gravityz2);
     oled.display();   
 } 
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Send request and read data from the module 3

byte msg3 [] = { 
                 3    
               };

digitalWrite (ENABLE_PIN, HIGH); 
delay(1);
sendMsg (fWrite, msg3, sizeof msg3);  

digitalWrite (ENABLE_PIN, LOW); 
 for (int i=0; i<14; i++) {

                      byte buf [9];
                      byte received = recvMsg (fAvailable, fRead, buf, sizeof buf);

                      if (received)
                      {
                        if (buf [0] != 3)
                          return; 
                        if(buf [0] == 3)
                        {
                          
                                temperature3    = (int)(((unsigned)buf [2] << 8) | buf [1]);
                                Gravityx3       = (int)(((unsigned)buf [4] << 8) | buf [3]);
                                Gravityy3       = (int)(((unsigned)buf [6] << 8) | buf [5]);
                                Gravityz3       = (int)(((unsigned)buf [8] << 8) | buf [7]);

                                temperature3    = (21.00 +(((float)temperature3)/8));
                                Gravityx3      *= conversionFactor;
                                Gravityy3      *= conversionFactor;
                                Gravityz3      *= conversionFactor;

                                GravityxOff3 = Gravityx3 - (-0.099685);
                                GravityyOff3 = Gravityy3 - (0.011565);
                                GravityzOff3 = Gravityz3 - (-0.075143);

                                GravityxCal3 =  0.999795 * GravityxOff3 + 0.008241 * GravityyOff3 + 0.015462 * GravityzOff3;
                                GravityyCal3 =  0.008241 * GravityxOff3 + 1.008767 * GravityyOff3 - 0.013595* GravityzOff3;
                                GravityzCal3 =  0.015462 * GravityxOff3 - 0.013595 * GravityyOff3 + 0.958726* GravityzOff3;

                                TotalG3 = sqrt ((GravityxCal3*GravityxCal3)+(GravityyCal3*GravityyCal3)+(GravityzCal3*GravityzCal3));
                                              
                     
                       ceck = 0;
                    
                      }
                      }
                   }
                  
 if (ceck == 0)
 {
    down1 = 0;
    digitalWrite(ledmodul3, HIGH);
    oled.clearDisplay();
    oled.setCursor(0,0);
    oled.println("Module 3: OK");
    oled.setTextSize(1);
    oled.setTextColor(SSD1306_WHITE);
    oled.print ("Gx: ");
    oled.println(Gravityx3);
    oled.print ("Gy: ");
    oled.print(Gravityy3);
    oled.print("  ");
    oled.print("TG: ");
    oled.println (TotalG3);
    oled.print ("Gz: ");
    oled.println(Gravityz3);
    oled.display();
 } 
 
 oled.setTextSize(2);
 oled.setTextColor(SSD1306_WHITE);

 if (ceck == 1 || TotalG3 < 0.5 && TotalG3 > 1.5) 
 {
     down1 = 1;
     digitalWrite(ledmodul3, LOW);
     oled.clearDisplay();
     oled.setCursor(0,0);
     oled.print("Module 3: KO");
     oled.setTextSize(1);
     oled.setTextColor(SSD1306_WHITE);
     oled.print ("Gx: ");
     oled.println(Gravityx3);
     oled.print ("Gy: ");
     oled.print(Gravityy3);
     oled.print("  ");
     oled.print("TG: ");
     oled.println (TotalG3);
     oled.print ("Gz: ");
     oled.println(Gravityz3);
     oled.display();   
 } 

 ceck = 1;   
 delay(delaybetween);
 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Send request and read data from the module 4
         
byte msg4 [] = { 
                 4    
               };

digitalWrite (ENABLE_PIN, HIGH); 
delay(1);
sendMsg (fWrite, msg4, sizeof msg4);

digitalWrite (ENABLE_PIN, LOW);  

 for (int i=0; i<14; i++) {

                      byte buf [9];
                      byte received = recvMsg (fAvailable, fRead, buf, sizeof buf);

                      if (received)
                      {
                        if (buf [0] != 4)
                          return; 
                        if(buf [0] == 4)
                        {
                          
                                temperature4    = (int)(((unsigned)buf [2] << 8) | buf [1]);
                                Gravityx4       = (int)(((unsigned)buf [4] << 8) | buf [3]);
                                Gravityy4       = (int)(((unsigned)buf [6] << 8) | buf [5]);
                                Gravityz4       = (int)(((unsigned)buf [8] << 8) | buf [7]);

                                temperature4    = (21.00 +(((float)temperature4)/8));
                                
                                Gravityx4      *= conversionFactor;
                                Gravityy4      *= conversionFactor;
                                Gravityz4      *= conversionFactor;

                                GravityxOff4 = Gravityx4 - (-0.099535);
                                GravityyOff4 = Gravityy4 - (0.006597);
                                GravityzOff4 = Gravityz4 - (-0.085722);

                                GravityxCal4 =  0.981895 * GravityxOff4 + 0.001914 * GravityyOff4 + 0.006723 * GravityzOff4;
                                GravityyCal4 =  0.001914 * GravityxOff4 + 0.983746 * GravityyOff4 + 0.110863 * GravityzOff4;
                                GravityzCal4 =  0.006723 * GravityxOff4 + 0.110863 * GravityyOff4 + 0.937835 * GravityzOff4;

                                TotalG4 = sqrt ((GravityxCal4*GravityxCal4)+(GravityyCal4*GravityyCal4)+(GravityzCal4*GravityzCal4));
                                              

                     
                       ceck = 0;
                    
                      }
                      }
                   }
if (ceck == 0)
 {
    down1 = 0;
    digitalWrite(ledmodul4, HIGH);
    oled.clearDisplay();
    oled.setCursor(0,0);
    oled.println("Module 4: OK");
    oled.setTextSize(1);
    oled.setTextColor(SSD1306_WHITE);
    oled.print ("Gx: ");
    oled.println(Gravityx4);
    oled.print ("Gy: ");
    oled.print(Gravityy4);
    oled.print("  ");
    oled.print("TG: ");
    oled.println (TotalG4);
    oled.print ("Gz: ");
    oled.println(Gravityz4);
    oled.display();
 } 
 
 oled.setTextSize(2);
 oled.setTextColor(SSD1306_WHITE);

 if (ceck == 1 || TotalG4 < 0.5 && TotalG4 > 1.5) 
 {
     down1 = 1;
     digitalWrite(ledmodul4, LOW);
     oled.clearDisplay();
     oled.setCursor(0,0);
     oled.print("Module 4: KO");
     oled.setTextSize(1);
     oled.setTextColor(SSD1306_WHITE);
     oled.print ("Gx: ");
     oled.println(Gravityx4);
     oled.print ("Gy: ");
     oled.print(Gravityy4);
     oled.print("  ");
     oled.print("TG: ");
     oled.println(TotalG4);
     oled.print ("Gz: ");
     oled.println(Gravityz4);
     oled.display();   
 } 

 ceck = 1;   
 delay(delaybetween);
                                        
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Send request and read data from the module 5

byte msg5 [] = { 
                 5    
               };

digitalWrite (ENABLE_PIN, HIGH); 
delay(1);
sendMsg (fWrite, msg5, sizeof msg5);

digitalWrite (ENABLE_PIN, LOW);  

 for (int i=0; i<14; i++) {
                      byte buf [9];
                      byte received = recvMsg (fAvailable, fRead, buf, sizeof buf);

                      if (received)
                      {
                        if (buf [0] != 5)
                          return; 
                        if(buf [0] == 5)
                        {
                          
                                temperature5    = (int)(((unsigned)buf [2] << 8) | buf [1]);
                                Gravityx5       = (int)(((unsigned)buf [4] << 8) | buf [3]);
                                Gravityy5       = (int)(((unsigned)buf [6] << 8) | buf [5]);
                                Gravityz5       = (int)(((unsigned)buf [8] << 8) | buf [7]);

                                temperature5    = (21.00 +(((float)temperature5)/8));
                                
                                Gravityx5      *= conversionFactor;
                                Gravityy5      *= conversionFactor;
                                Gravityz5      *= conversionFactor;

                                GravityxOff5 = Gravityx5 - (-0.121030);
                                GravityyOff5 = Gravityy5 - (0.007038);
                                GravityzOff5 = Gravityz5 - (-0.008357);

                                GravityxCal5 =  1.020314 * GravityxOff5 + 0.018132 * GravityyOff5 - 0.006800 * GravityzOff5;
                                GravityyCal5 =  0.018132 * GravityxOff5 + 1.001539 * GravityyOff5 - 0.008729 * GravityzOff5;
                                GravityzCal5 = -0.006800 * GravityxOff5 - 0.008729 * GravityyOff5 + 0.971765 * GravityzOff5;

                                TotalG5 = sqrt ((GravityxCal5*GravityxCal5)+(GravityyCal5*GravityyCal5)+(GravityzCal5*GravityzCal5));

                     
                       ceck = 0;
                                                                  
                    
                      }
                      }
                      
  }
                   
 if (ceck == 0)
 {
    down1 = 0;
    digitalWrite(ledmodul5, HIGH);
    oled.clearDisplay();
    oled.setCursor(0,0);
    oled.println("Module 5: OK");
    oled.setTextSize(1);
    oled.setTextColor(SSD1306_WHITE);
    oled.print ("Gx: ");
    oled.println(Gravityx5);
    oled.print ("Gy: ");
    oled.print(Gravityy5);
    oled.print("  ");
    oled.print("TG: ");
    oled.println (TotalG5);
    oled.print ("Gz: ");
    oled.println(Gravityz5);
    oled.display();
 } 
 
 oled.setTextSize(2);
 oled.setTextColor(SSD1306_WHITE);

 if (ceck == 1 || TotalG5 < 0.5 && TotalG5 > 1.5) 
 {
     down1 = 1;
     digitalWrite(ledmodul5, LOW);
     oled.clearDisplay();
     oled.setCursor(0,0);
     oled.print("Module 5: KO");
     oled.setTextSize(1);
     oled.setTextColor(SSD1306_WHITE);
     oled.print ("Gx: ");
     oled.println(Gravityx5);
     oled.print ("Gy: ");
     oled.print(Gravityy5);
     oled.print("  ");
     oled.print("TG: ");
     oled.println (TotalG5);
     oled.print ("Gz: ");
     oled.println(Gravityz5);
     oled.display();   
 } 

 ceck = 1;   
 delay(delaybetween);
                           
  oled.setTextSize(1);
  oled.setTextColor(SSD1306_WHITE);
                              

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Derive the angulars values


                                              //Derive pitch data
                                              pitch1 = atan2 (-GravityxCal1, sqrt ((GravityyCal1*GravityyCal1)+(GravityzCal1*GravityzCal1)));
                                              pitch2 = atan2 (-GravityxCal2, sqrt ((GravityyCal2*GravityyCal2)+(GravityzCal2*GravityzCal2)));
                                              pitch3 = atan2 (-GravityxCal3, sqrt ((GravityyCal3*GravityyCal3)+(GravityzCal3*GravityzCal3)));
                                              pitch4 = atan2 (-GravityxCal4, sqrt ((GravityyCal4*GravityyCal4)+(GravityzCal4*GravityzCal4)));
                                              pitch5 = atan2 (-GravityxCal5, sqrt ((GravityyCal5*GravityyCal5)+(GravityzCal5*GravityzCal5)));
                                              //Derive roll data
                                              roll1  = atan2 (-GravityyCal1, sqrt ((GravityxCal1*GravityxCal1)+(GravityzCal1*GravityzCal1)));
                                              roll2  = atan2 (-GravityyCal2, sqrt ((GravityxCal2*GravityxCal2)+(GravityzCal2*GravityzCal2)));
                                              roll3  = atan2 (-GravityyCal3, sqrt ((GravityxCal3*GravityxCal3)+(GravityzCal3*GravityzCal3)));
                                              roll4  = atan2 (-GravityyCal4, sqrt ((GravityxCal4*GravityxCal4)+(GravityzCal4*GravityzCal4)));
                                              roll5  = atan2 (-GravityyCal5, sqrt ((GravityxCal5*GravityxCal5)+(GravityzCal5*GravityzCal5)));
                                              //Derive tilt data
                                              tilt1 = atan2 (sqrt ((GravityxCal1*GravityxCal1)+(GravityyCal1*GravityyCal1)), GravityzCal1); 
                                              tilt2 = atan2 (sqrt ((GravityxCal2*GravityxCal2)+(GravityyCal2*GravityyCal2)), GravityzCal2);
                                              tilt3 = atan2 (sqrt ((GravityxCal3*GravityxCal3)+(GravityyCal3*GravityyCal3)), GravityzCal3);
                                              tilt4 = atan2 (sqrt ((GravityxCal4*GravityxCal4)+(GravityyCal4*GravityyCal4)), GravityzCal4);
                                              tilt5 = atan2 (sqrt ((GravityxCal5*GravityxCal5)+(GravityyCal5*GravityyCal5)), GravityzCal5);
                                              
                                              tilt1 *= 180.0 / PI;
                                              tilt2 *= 180.0 / PI;
                                              tilt3 *= 180.0 / PI;
                                              tilt4 *= 180.0 / PI;
                                              tilt5 *= 180.0 / PI;

                                              pitch1 *= 180.0 / PI;
                                              pitch2 *= 180.0 / PI;
                                              pitch3 *= 180.0 / PI;
                                              pitch4 *= 180.0 / PI;
                                              pitch5 *= 180.0 / PI;
                                              
                                              roll1 *= 180.0 / PI;
                                              roll2 *= 180.0 / PI;
                                              roll3 *= 180.0 / PI;
                                              roll4 *= 180.0 / PI;
                                              roll5 *= 180.0 / PI;
                                              
                                              //First start setting as zero position
                                              if(misurazero == 0)
                                              { 
                                                tiltzero1 = tilt1;
                                                tiltzero2 = tilt2;
                                                tiltzero3 = tilt3;
                                                tiltzero4 = tilt4;
                                                tiltzero5 = tilt5;

                                                pitchzero1 = pitch1;
                                                pitchzero2 = pitch2;
                                                pitchzero3 = pitch3;
                                                pitchzero4 = pitch4;
                                                pitchzero5 = pitch5;

                                                rollzero1 = roll1;
                                                rollzero2 = roll2;
                                                rollzero3 = roll3;
                                                rollzero4 = roll4;
                                                rollzero5 = roll5;
                                                misurazero = 1;
                         
                                              }

                                              //Subctract the zero position
                                              pitch1 -= pitchzero1;
                                              pitch2 -= pitchzero2;
                                              pitch3 -= pitchzero3;
                                              pitch4 -= pitchzero4;
                                              pitch5 -= pitchzero5;
                                              
                                              roll1  -= rollzero1;
                                              roll2  -= rollzero2;
                                              roll3  -= rollzero3;
                                              roll4  -= rollzero4;
                                              roll5  -= rollzero5;

                                              tilt1 -=tiltzero1;
                                              tilt2 -= tiltzero2;
                                              tilt3 -= tiltzero3;
                                              tilt4 -= tiltzero4;
                                              tilt5 -= tiltzero5;
                                              
                                              
                                            
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// Derive directions of the tilt
 
 if (pitch1 >0 && roll1 >0)
 {
  Direction1 = 360 -((atan(roll1/pitch1)*(180.0/PI)));
 
 }
 
 if (pitch1 >0 && roll1<0)
{
  Direction1 = 360-((360+(atan(roll1/pitch1)*(180.0/PI))));

}

 if (pitch1< 0 && roll1>0)
{
 Direction1 = (180-(atan(roll1/pitch1)*(180.0/PI)));

}
 
 if (pitch1 < 0 && roll1 < 0)
 {
  Direction1 = (180 -(atan(roll1/pitch1)*(180.0/PI)));

 }


 if (pitch2 >0 && roll2 >0)
 {
  Direction2 = 360 -((atan(roll2/pitch2)*(180.0/PI)));
 
 }
 if (pitch2 >0 && roll2<0)
{
  Direction2 = 360-((360+(atan(roll2/pitch2)*(180.0/PI))));
 
}
 
 if (pitch2< 0 && roll2>0)
{
 Direction2 = (180-(atan(roll2/pitch2)*(180.0/PI)));

}
 
 if (pitch2 < 0 && roll2 < 0)
 {
  Direction2 = (180 -(atan(roll2/pitch2)*(180.0/PI)));
 
 }


 if (pitch3 >0 && roll3 >0)
 {
  Direction3 = 360 -((atan(roll3/pitch3)*(180.0/PI)));
  
 }
 if (pitch3 >0 && roll3<0)
{
  Direction3 = 360-((360+(atan(roll3/pitch3)*(180.0/PI))));
 
}
 if (pitch3< 0 && roll3>0)
{
 Direction3 = (180-(atan(roll3/pitch3)*(180.0/PI)));

}
 
 if (pitch3 < 0 && roll3 < 0)
 {
  Direction3 = (180 -(atan(roll3/pitch3)*(180.0/PI)));
  
 }


 if (pitch4 >0 && roll4 >0)
 {
  Direction4 = 360 -((atan(roll4/pitch4)*(180.0/PI)));

 }
 if (pitch4 >0 && roll4<0)
{
  Direction4 = 360-((360+(atan(roll4/pitch4)*(180.0/PI))));
 
}
 
 if (pitch4< 0 && roll4>0)
{
 Direction4 = (180-(atan(roll4/pitch4)*(180.0/PI)));
 
}
 
 if (pitch1 < 0 && roll1 < 0)
 {
  Direction4 = (180 -(atan(roll4/pitch4)*(180.0/PI)));

 }


 if (pitch5 >0 && roll5 >0)
 {
  Direction5 = 360 -((atan(roll5/pitch5)*(180.0/PI)));
  
 }
 if (pitch5 >0 && roll5<0)
{
  Direction5 = 360-((360+(atan(roll5/pitch5)*(180.0/PI))));
  
}
 
 if (pitch5< 0 && roll5>0)
{
 Direction5 = (180-(atan(roll5/pitch5)*(180.0/PI)));

}
 
 if (pitch5 < 0 && roll5 < 0)
 {
  Direction5 = (180 -(atan(roll5/pitch5)*(180.0/PI)));
  
 }                                         

 /////////////////////////////////////////////////////////////////////////////////////////////////////////Assign the verse of the tilt according the direction
 
                                              if(Direction1 > 0 && Direction1<90)
                                                 {tilt1 = abs(tilt1);}
                                              if(Direction1 > 90 && Direction1<180)
                                                 {tilt1 = abs(tilt1)*-1;}
                                              if(Direction1 > 180 && Direction1<270)
                                                 {tilt1 = abs(tilt1)*-1;}
                                              if(Direction1 >270 && Direction1<360)
                                                 {tilt1 = abs(tilt1);}

                                              if(Direction2 > 0 && Direction2<90)
                                                 {tilt2 = abs(tilt2);}
                                              if(Direction2 > 90 && Direction2<180)
                                                 {tilt2 = abs(tilt2)*-1;}
                                              if(Direction2 > 180 && Direction2<270)
                                                 {tilt2 = abs(tilt2)*-1;}
                                              if(Direction2 > 270 && Direction2<360)
                                                 {tilt2 = abs(tilt2);}

                                              if(Direction3 > 0 && Direction3<90)
                                                 {tilt3 = abs(tilt3);}
                                              if(Direction3 > 90 && Direction3<180)
                                                 {tilt3 = abs(tilt3)*-1;}
                                              if(Direction3 > 180 && Direction3<270)
                                                 {tilt3 = abs(tilt3)*-1;}
                                              if(Direction3 > 270 && Direction3<360)
                                                 {tilt3 = abs(tilt3);}

                                              if(Direction4 > 0 && Direction4<90)
                                                 {tilt4 = abs(tilt4);}
                                              if(Direction4 > 90 && Direction4<180)
                                                 {tilt4 = abs(tilt4)*-1;}
                                              if(Direction4 > 180 && Direction4<270)
                                                 {tilt4 = abs(tilt4)*-1;}
                                              if(Direction4 > 270 && Direction4<360)
                                                 {tilt4 = abs(tilt4);}

                                              if(Direction5 > 0 && Direction5<90)
                                                 {tilt5 = abs(tilt5);}
                                              if(Direction5 > 90 && Direction5<180)
                                                 {tilt5 = abs(tilt5)*-1;}
                                              if(Direction5 > 180 && Direction5<270)
                                                 {tilt5 = abs(tilt5)*-1;}
                                              if(Direction5 > 270 && Direction5<360)
                                                 {tilt5 = abs(tilt5);}

////////////////////////////////////////////////////////////////////////////////////////////////////////

                                              //Project the real displacement on the X axis of the modules
                                              resultantx1 = cos (Direction1 * (PI/180.0)); 
                                              resultantx2 = cos (Direction2 * (PI/180.0));
                                              resultantx3 = cos (Direction3 * (PI/180.0));
                                              resultantx4 = cos (Direction4 * (PI/180.0));
                                              resultantx5 = cos (Direction5 * (PI/180.0));

                                              resultantx1 = resultantx1 * abs(tilt1);
                                              resultantx2 = resultantx2 * abs(tilt2);
                                              resultantx3 = resultantx3 * abs(tilt3);
                                              resultantx4 = resultantx4 * abs(tilt4);
                                              resultantx5 = resultantx5 * abs(tilt5);
                                              
                                              displacement1 =  sin (resultantx1 * (PI/180.0)); 
                                              displacement2 =  sin (resultantx2 * (PI/180.0));
                                              displacement3 =  sin (resultantx3 * (PI/180.0));
                                              displacement4 =  sin (resultantx4 * (PI/180.0));
                                              displacement5 =  sin (resultantx5 * (PI/180.0));

                                              displacement1 =  sin (tilt1 * (PI/180.0));
                                              displacement2 =  sin (tilt2 * (PI/180.0));
                                              displacement3 =  sin (tilt3 * (PI/180.0));
                                              displacement4 =  sin (tilt4 * (PI/180.0));
                                              displacement5 =  sin (tilt5 * (PI/180.0));

                                              displacement1 *= lenghtmodule;
                                              displacement2 *= lenghtmodule;
                                              displacement3 *= lenghtmodule;
                                              displacement4 *= lenghtmodule;
                                              displacement5 *= lenghtmodule;
                                              
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// Derive the total displacement at the head of the inclinometer column
                                              
                                              totdisplacementx = displacement1 + displacement2 + displacement3 + displacement4 + displacement5;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////Save data on the SD card


                                              

                                                    if (!SD.begin(chipSelect)) {
                                                            sdceck = 0;
                                                          } else sdceck = 1;
                                                    DateTime now = rtc.now();
                                                    File dataFile = SD.open("datalog.txt", FILE_WRITE);

                                                    if (dataFile) {
                                                                    dataFile.print(now.hour(), DEC);
                                                                    dataFile.print(':');
                                                                    dataFile.print(now.minute(), DEC);
                                                                    dataFile.print(':');
                                                                    dataFile.print(now.second(), DEC);
                                                                    dataFile.print(" ");
                                                                    dataFile.print(now.day(), DEC);
                                                                    dataFile.print('/');
                                                                    dataFile.print(now.month(), DEC);
                                                                    dataFile.print('/');
                                                                    dataFile.print(now.year(), DEC);
                                                                    dataFile.print(", ");
                                                                    
                                                                    dataFile.print(temperature1);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(temperature2);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(temperature3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(temperature4);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(temperature5);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(tilt1,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(tilt2,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(tilt3,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(tilt4,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(tilt5,3);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(pitch1,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(pitch2,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(pitch3,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(pitch4,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(pitch5,3);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(roll1,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(roll2,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(roll3,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(roll4,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(roll5,3);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(Direction1,0);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Direction2,0);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Direction3,0);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Direction4,0);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Direction5,0);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print("Total displacement:");
                                                                    dataFile.print(", ");
                                                                    dataFile.print(totdisplacementx,3);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityx1,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityy1,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityz1,6);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(Gravityx2,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityy2,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityz2,6);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(Gravityx3,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityy3,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityz3,6);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(Gravityx4,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityy4,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityz4,6);
                                                                    dataFile.print(", ,");
                                                                    
                                                                    dataFile.print(Gravityx5,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.print(Gravityy5,6);
                                                                    dataFile.print(", ");
                                                                    dataFile.println(Gravityz5,6);
                                                                    dataFile.close();
 
                                                                  }
                                                                   
 //Displaying data on the OLED

  for (float t = 0; t<cicleread; t++) //Displaying data for a number of cycle
  {
    
   
    digitalWrite(read, HIGH);
                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 1 "); 
                        oled.print("TILT: "); 
                        oled.println(tilt1,3);
                        oled.print("PITCH: ");
                        oled.println(pitch1,3);
                        oled.print("ROLL: ");
                        oled.println(roll1,3);
                        oled.print("DIRECTION: ");
                        oled.print(Direction1,0);
                        oled.display();
                        delay (4000);
                        
                        
    digitalWrite(read, LOW);
    
                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 1");
                        oled.print("TG: ");
                        oled.println(TotalG1);
                        oled.print("TEMP C: ");
                        oled.println(temperature1);
                        oled.print("DISPL1 cm: ");
                        oled.println(totdisplacementx,1);
                        oled.print("SD: ");
                        if(sdceck ==1){oled.println("OK");} else oled.println("KO");
                        oled.display();
                        delay(4000);
                        
                        
  /////////////////////////////////////////////////////////7                      
    
    digitalWrite(read, HIGH);
    
                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 2 ");
                        oled.print("TILT: "); 
                        oled.println(tilt2,3);
                        oled.print("PITCH: ");
                        oled.println(pitch2,3);
                        oled.print("ROLL: ");
                        oled.println(roll2,3);
                        oled.print("DIRECTION: ");
                        oled.print(Direction2,0);
                        oled.display();
                        delay(4000);
                        

digitalWrite(read, LOW);

                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 2");
                        oled.print("TG: ");
                        oled.println(TotalG2);
                        oled.print("TEMP C: ");
                        oled.println(temperature2);
                        oled.print("DISPL2 cm: ");
                        oled.println(totdisplacementx,1);
                        oled.print("SD: ");
                        if(sdceck ==1){oled.println("OK");} else oled.println("KO");
                        oled.display();
                        delay(4000);
                        
digitalWrite(read, HIGH);

                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 3 ");
                        oled.print("TILT: "); 
                        oled.println(tilt3,3);
                        oled.print("PITCH: ");
                        oled.println(pitch3,3);
                        oled.print("ROLL: ");
                        oled.println(roll3,3);
                        oled.print("DIRECTION: ");
                        oled.print(Direction3,0);
                        oled.display();
                        delay(4000);
                       
                        
digitalWrite(read, LOW);

                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 3");
                        oled.print("TG: ");
                        oled.println(TotalG3);
                        oled.print("TEMP C: ");
                        oled.println(temperature3);
                        oled.print("DISPL3 cm: ");
                        oled.println(totdisplacementx,1);
                        oled.print("SD: ");
                        if(sdceck ==1){oled.println("OK");} else oled.println("KO");
                        oled.display();
                        delay(4000);
                        

digitalWrite(read, HIGH);

                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 4 ");
                        oled.print("TILT: "); 
                        oled.println(tilt4,3);
                        oled.print("PITCH: ");
                        oled.println(pitch4,3);
                        oled.print("ROLL: ");
                        oled.println(roll4,3);
                        oled.print("DIRECTION: ");
                        oled.print(Direction4,0);
                        oled.display();
                        delay(4000);
                        

 digitalWrite(read, LOW);
 
                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 4");
                        oled.print("TG: ");
                        oled.println(TotalG4);
                        oled.print("TEMP C: ");
                        oled.println(temperature4);
                        oled.print("DISPL4 cm: ");
                        oled.println(totdisplacementx,1);
                        oled.print("SD: ");
                        if(sdceck ==1){oled.println("OK");} else oled.println("KO");
                        oled.display();
                        delay(4000);
 

digitalWrite(read, HIGH);

                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 5 ");
                        oled.print("TILT: "); 
                        oled.println(tilt5,3);
                        oled.print("PITCH: ");
                        oled.println(pitch5,3);
                        oled.print("ROLL: ");
                        oled.println(roll5,3);
                        oled.print("DIRECTION: ");
                        oled.print(Direction5,0);
                        oled.display();
                        delay(4000);
    

digitalWrite(read, LOW);

                        oled.clearDisplay();
                        oled.setCursor(0,0); 
                        oled.println("MODULE 5");
                        oled.print("TG: ");
                        oled.println(TotalG5);
                        oled.print("TEMP C: ");
                        oled.println(temperature5);
                        oled.print("DISPL5 cm: ");
                        oled.println(totdisplacementx,1);
                        oled.print("SD: ");
                        if(sdceck ==1){oled.println("OK");} else oled.println("KO");
                        oled.display();
                        delay(4000);
                        
////////////////////////////////////////////////////////////////////////////////////////Displaying bubble level for each module tilt value
digitalWrite(read, HIGH);

                        oled.clearDisplay();
                        oled.drawCircle(96, 32, 25, WHITE);
                        oled.drawLine(96-32+7, 32, 96+32-7, 32, WHITE);
                        oled.drawLine(96, 7, 96, 64-7, WHITE);
                        oled.fillTriangle( 96-4, 7, 96+4, 7, 96, 0, WHITE);
                        oled.drawTriangle( 64+7, 32+4,  64+7, 32-4,  64, 32, WHITE);
                        by = mapf(pitch1, 45,-45, 7, 64-7);
                        bx = mapf(roll1, 45, -45, 64+7, 128-7);
                        oled.drawCircle(bx, by, 4, WHITE);
                        oled.setCursor(0, 0);
                        oled.println("TILT 1: "); 
                        oled.println(tilt1,2);
                        oled.println();
                        oled.println("DISPL1 cm: ");
                        oled.println(displacement1,1);
                        oled.display();
                        delay(4000);

digitalWrite(read, LOW);

                        oled.clearDisplay();
                        oled.drawCircle(96, 32, 25, WHITE);
                        oled.drawLine(96-32+7, 32, 96+32-7, 32, WHITE);
                        oled.drawLine(96, 7, 96, 64-7, WHITE);
                        oled.fillTriangle( 96-4, 7, 96+4, 7, 96, 0, WHITE);
                        oled.drawTriangle( 64+7, 32+4,  64+7, 32-4,  64, 32, WHITE);
                        by = mapf(pitch2, 45,-45, 7, 64-7);
                        bx = mapf(roll2, 45, -45, 64+7, 128-7);
                        oled.drawCircle(bx, by, 4, WHITE);
                        oled.setCursor(0, 0);
                        oled.println("TILT 2: "); 
                        oled.println(tilt2,2);
                        oled.println();
                        oled.println("DISPL2 cm: ");
                        oled.println(displacement2,1);
                        oled.display();
                        delay(4000);

digitalWrite(read, HIGH);

                        oled.clearDisplay();
                        oled.drawCircle(96, 32, 25, WHITE);
                        oled.drawLine(96-32+7, 32, 96+32-7, 32, WHITE);
                        oled.drawLine(96, 7, 96, 64-7, WHITE);
                        oled.fillTriangle( 96-4, 7, 96+4, 7, 96, 0, WHITE);
                        oled.drawTriangle( 64+7, 32+4,  64+7, 32-4,  64, 32, WHITE);
                        by = mapf(pitch3, 45,-45, 7, 64-7);
                        bx = mapf(roll3, 45, -45, 64+7, 128-7);
                        oled.drawCircle(bx, by, 4, WHITE);
                        oled.setCursor(0, 0);
                        oled.println("TILT 3: "); 
                        oled.println(tilt3,2);
                        oled.println();
                        oled.println("DISPL3 cm: ");
                        oled.println(displacement3,1);
                        oled.display();
                        delay(4000);
digitalWrite(read, LOW);

                        oled.clearDisplay();
                        oled.drawCircle(96, 32, 25, WHITE);
                        oled.drawLine(96-32+7, 32, 96+32-7, 32, WHITE);
                        oled.drawLine(96, 7, 96, 64-7, WHITE);
                        oled.fillTriangle( 96-4, 7, 96+4, 7, 96, 0, WHITE);
                        oled.drawTriangle( 64+7, 32+4,  64+7, 32-4,  64, 32, WHITE);
                        by = mapf(pitch4, 45,-45, 7, 64-7);
                        bx = mapf(roll4, 45, -45, 64+7, 128-7);
                        oled.drawCircle(bx, by, 4, WHITE);
                        oled.setCursor(0, 0);
                        oled.println("TILT 4: "); 
                        oled.println(tilt4,2);
                        oled.println();
                        oled.println("DISPL4 cm: ");
                        oled.println(displacement4,1);
                        oled.display();
                        delay(4000);
                    

digitalWrite(read, HIGH);

                        oled.clearDisplay();
                        oled.drawCircle(96, 32, 25, WHITE);
                        oled.drawLine(96-32+7, 32, 96+32-7, 32, WHITE);
                        oled.drawLine(96, 7, 96, 64-7, WHITE);
                        oled.fillTriangle( 96-4, 7, 96+4, 7, 96, 0, WHITE);
                        oled.drawTriangle( 64+7, 32+4,  64+7, 32-4,  64, 32, WHITE);
                        by = mapf(pitch5, 45,-45, 7, 64-7);
                        bx = mapf(roll5, 45, -45, 64+7, 128-7);
                        oled.drawCircle(bx, by, 4, WHITE);
                        oled.setCursor(0, 0);
                        oled.println("TILT 5: "); 
                        oled.println(tilt5,2);
                        oled.println();
                        oled.println("DISPL5 cm: ");
                        oled.println(displacement5,1);
                        oled.display();
                        delay(4000);

digitalWrite(read, LOW);

                        oled.setTextSize(2);
                        oled.setTextColor(SSD1306_WHITE);
                        oled.clearDisplay();
                        oled.setCursor(0,0);
                        oled.println("DISPLX cm:"); 
                        oled.println(totdisplacementx,1);
                        oled.display();
                        delay(4000);
                        oled.setTextSize(1);
                        oled.setTextColor(SSD1306_WHITE);
                                                                    
}

digitalWrite(ledmodul1, LOW);
digitalWrite(ledmodul2, LOW);
digitalWrite(ledmodul3, LOW);
digitalWrite(ledmodul4, LOW);
digitalWrite(ledmodul5, LOW);
  
  }
  
float mapf(float x, float in_min, float in_max, float out_min, float out_max) //Mapping tilt value for range of bubble level
{
  return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
}
