/*
  Tilt module software ver 1.0 

  This sketch reads the accelerometer sensor LSM9DS0, apply thermal compensation, the Kalman filter 
  and sending the data to the Master Sation via RS 485 comunication board 
  The software is coded and compliled with Arduino IDE https://www.arduino.cc/en/Main/Software
  
  Used electronic devices :
 
 - Arduino NANO  https://store.arduino.cc/arduino-nano
 - RS 485 board 
 - LSM9DS0 sensor
 
  created 10 May 2020
  by Giuseppe Ruzza
  
  Department of Science and Technology 
  University of Sannio, Benevento, Italy.
*/

#include "RS485_protocol.h"   //Communication library        http://www.gammon.com.au/forum/?id=11428
#include <Adafruit_LSM9DS0.h> //LSM9DS0 Adafruit library     https://github.com/adafruit/Adafruit_LSM9DS0_Library
#include <Adafruit_Sensor.h>  //Adafruit sensors library     https://github.com/adafruit/Adafruit_Sensor
#include <Wire.h>             //I2C communication library

Adafruit_LSM9DS0 lsm = Adafruit_LSM9DS0();

//enable pin of the communication RS485 board
int  ENABLE_PIN = 9;
int address = 1; //address of the module 

//Sensor temperature data
int temperature;

//Raw accelerations data
int X_AXIS_A;
int Y_AXIS_A;
int Z_AXIS_A;

//Sensor temperature value
float temperatureinternal;

//Kalman filter variables
float Q  = 0.0001;  
float Rx = 7000;
float Ry = 7000;
float Rz = 7000;

float Pcx = 0.0;
float Gx = 0.0;
float Px = 1.0;
float Xpx = 0.0;
float Zpx = 0.0;
float Xraw = 0.0;

float Pcy = 0.0;
float Gy = 0.0;
float Py = 1.0;
float Xpy = 0.0;
float Zpy = 0.0;
float Yraw = 0.0;

float Pcz = 0.0;
float Gz = 0.0;
float Pz = 1.0;
float Xpz = 0.0;
float Zpz = 0.0;
float Zraw = 0.0;

//Temperature variable
float x;
//Thermal compensate raw accelerometric data
float Xrawc;
float Yrawc;
float Zrawc;

//Thermal compensated bias 
double errore_Y;
double errore_X;
double errore_Z;


void setup()
{
  lsm.begin();  //Sensor library inizaization
  lsm.setupAccel(lsm.LSM9DS0_ACCELRANGE_2G); //Sensor accelerometer range setting
  Serial.begin(9600); //Communication inizialization
  //Pin setting for comunication board
  pinMode(ENABLE_PIN, OUTPUT);
  digitalWrite(ENABLE_PIN, LOW); 
}

void loop()
{
  //Receiving request from communication bus
  byte buf [1];
  byte received = recvMsg (fAvailable, fRead, buf, sizeof (buf)); 
  
  if (received) // If a request arrived, check its address
    {
    if (buf [0] != address) //If address don't correspond to the module return
      return; 
    if(buf [0] == address)  //If address corresponds to the module read sensor and sending data to Master Station
    {
             
              lsm.read();                                               
              Px = (int)lsm.accelData.x;
              Py = (int)lsm.accelData.y;
              Pz = (int)lsm.accelData.z;

                                         for (int c = 0; c < 1000; c++) //Repeat the Kalman filter for 1000 loop befor send data
                                                            
                                                            {
                                                              lsm.read();
                                                              //read accelerometric data
                                                              X_AXIS_A = (int)lsm.accelData.x;
                                                              Y_AXIS_A = (int)lsm.accelData.y;
                                                              Z_AXIS_A = (int)lsm.accelData.z;
                                                           
                                                              temperatureinternal =   21.00 +(((float)lsm.temperature)/8);
                                                            
                                                              x = temperatureinternal;
                                                              //derive the error from the polynomial equations see [1]
                                                              errore_Z =  642.8 + -11.88*x + -0.03703*Z_AXIS_A    +  0.05491*pow(x,2)  +  0.0001487*x*Z_AXIS_A  +  1.714e-06*pow(Z_AXIS_A,2);
                                                              errore_X =  682.6 + -24.44*x + -0.001053*X_AXIS_A   + -0.08801*pow(x,2)  +  3.975e-06*x*X_AXIS_A  + -2.647e-07*pow(X_AXIS_A,2);
                                                              errore_Y = -467.5 +  19.3*x  +  0.0003001*Y_AXIS_A  + -0.01975*pow(x,2)  + -1.846e-05*x*Y_AXIS_A  +  3.804e-08*pow(Y_AXIS_A,2);
                                                              //remouve the thermal bias from the accelerometric data
                                                              Xrawc = X_AXIS_A-errore_X;
                                                              Yrawc = Y_AXIS_A-errore_Y;
                                                              Zrawc = Z_AXIS_A-errore_Z;                                        
                                                              //Kalman filter X axis
                                                              Pcx = Px + Q;
                                                              Gx = Pcx/(Pcx + Rx);    // kalman gain
                                                              Px = (1-Gx)*Pcx;
                                                              Xpx = Xraw;
                                                              Zpx = Xpx;
                                                              Xraw = Gx*((Xrawc)-Zpx)+Xpx;
                                                              //Kalman filter Y axis
                                                              Pcy = Py + Q;
                                                              Gy = Pcy/(Pcy + Ry);    // kalman gain
                                                              Py = (1-Gy)*Pcy;
                                                              Xpy = Yraw;
                                                              Zpy = Xpy;
                                                              Yraw = Gy*((Yrawc)-Zpy)+Xpy;
                                                              //Kalman filter Z axis
                                                              Pcz = Pz + Q;
                                                              Gz = Pcz/(Pcz + Rz);    // kalman gain
                                                              Pz = (1-Gz)*Pcz;
                                                              Xpz = Zraw;
                                                              Zpz = Xpz;
                                                              Zraw = Gz*((Zrawc)-Zpz)+Xpz;
                                                        }

                                                         // prepare variables for the seding message 
                                                          temperature = int(lsm.temperature);
                                                          unsigned char temperaturelsb = (unsigned)temperature & 0xff; 
                                                          unsigned char temperaturemsb = (unsigned)temperature >> 8; 
                                                         
                                                          int Xrawint = int(Xraw);
                                                          unsigned char Xrawlsb = (unsigned)Xrawint & 0xff; 
                                                          unsigned char Xrawmsb = (unsigned)Xrawint >> 8; 
                                                         
                                                          int Yrawint = int(Yraw);
                                                          unsigned char Yrawlsb = (unsigned)Yrawint & 0xff; 
                                                          unsigned char Yrawmsb = (unsigned)Yraw >> 8; 
                                                         
                                                          int Zrawint = int(Zraw);
                                                          unsigned char Zrawlsb = (unsigned)Zrawint & 0xff; 
                                                          unsigned char Zrawmsb = (unsigned)Zrawint >> 8; 
                                     
                                     //Message to send to Master Station
                                     char msg  [] = 
                                     {
                                     address,
                                     temperaturelsb,
                                     temperaturemsb, 
                                     Xrawlsb,  
                                     Xrawmsb,  
                                     Yrawlsb,
                                     Yrawmsb,
                                     Zrawlsb,
                                     Zrawmsb
                                  }; 

                                  
    delay (1);  // Give the master a moment to prepare to receive
    digitalWrite (ENABLE_PIN, HIGH);  // enable sending
    
    sendMsg (fWrite, msg, sizeof msg); //sending messagge
    
    digitalWrite (ENABLE_PIN, LOW);  // disable sending
 
    }

    }
    
   
 } //End of loop
   

//Communication functions
void fWrite (const byte what)
  {
   Serial.write (what);
   Serial.flush(); 
  }
  
int fAvailable ()
  {
  return Serial.available ()>0;  
  }

int fRead ()
  {
  return Serial.read ();  
  }

//[1] Ruzza, G.; Guerriero, L.; Revellino, P.; Guadagno, F.M. Thermal Compensation of Low-Cost MEMS Accelerometers for Tilt Measurements. Sensors 2018, 18, 2536.
